@@ -11,8 +11,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.2-branch/Approximate_min_ellipsoid_d/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation.h $
-// $Id: Khachiyan_approximation.h 28567 2006-02-16 14:30:13Z lsaboret $
+// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.3-branch/Approximate_min_ellipsoid_d/include/CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation.h $
+// $Id: Khachiyan_approximation.h 36397 2007-02-16 15:56:11Z gaertner $
 // 
 //
 // Author(s)     : Kaspar Fischer <fischerk@inf.ethz.ch>
@@ -626,6 +626,6 @@ namespace CGAL {
 
 }
 
-#include <CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation.C>
+#include <CGAL/Approximate_min_ellipsoid_d/Khachiyan_approximation_impl.h>
 
 #endif // CGAL_KHACHIYAN_APPROXIMATION_H
