@@ -11,8 +11,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.2-branch/Arrangement_2/include/CGAL/Arr_curve_data_traits_2.h $
-// $Id: Arr_curve_data_traits_2.h 28567 2006-02-16 14:30:13Z lsaboret $
+// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.3-branch/Arrangement_2/include/CGAL/Arr_curve_data_traits_2.h $
+// $Id: Arr_curve_data_traits_2.h 35514 2006-12-11 15:34:13Z wein $
 // 
 //
 // Author(s)     : Ron Wein          <wein@post.tau.ac.il>
@@ -41,7 +41,8 @@ CGAL_BEGIN_NAMESPACE
  * the overlapping subcurve is obtained from the merge functor.
  * All other functors are inherited from the base ordinary traits class.
  */
-template <class Traits_, class XMonotoneCurveData_, class Merge_,
+template <class Traits_, class XMonotoneCurveData_, 
+          class Merge_ = _Default_merge_func<XMonotoneCurveData_>,
           class CurveData_ = XMonotoneCurveData_,
           class Convert_ = _Default_convert_func<CurveData_,
                                                  XMonotoneCurveData_> >
@@ -59,7 +60,9 @@ public:
   typedef typename Base_traits_2::X_monotone_curve_2  Base_x_monotone_curve_2;
   typedef typename Base_traits_2::Point_2             Point_2;
 
-  typedef typename Base_traits_2::Has_left_category   Has_left_category;
+  typedef typename Base_traits_2::Has_left_category      Has_left_category;
+  typedef typename Base_traits_2::Has_boundary_category  Has_boundary_category;
+
   typedef typename Base_traits_2::Has_merge_category  Base_has_merge_category;
   typedef Tag_true                                    Has_merge_category;
 
