@@ -1,4 +1,4 @@
-// Copyright (c) 1998-2004  Utrecht University (The Netherlands),
+// Copyright (c) 1998-2007  Utrecht University (The Netherlands),
 // ETH Zurich (Switzerland), Freie Universitaet Berlin (Germany),
 // INRIA Sophia-Antipolis (France), Martin-Luther-University Halle-Wittenberg
 // (Germany), Max-Planck-Institute Saarbruecken (Germany), RISC Linz (Austria),
@@ -15,31 +15,36 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.2-branch/Interval_arithmetic/include/CGAL/FPU.h $
-// $Id: FPU.h 31497 2006-06-09 17:58:12Z spion $
-// 
+// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.3-branch/Number_types/include/CGAL/FPU.h $
+// $Id: FPU.h 37955 2007-04-05 13:02:19Z spion $
+//
 //
 // Author(s)     : Sylvain Pion
 
 #ifndef CGAL_FPU_H
 #define CGAL_FPU_H
 
+#include <CGAL/number_type_basic.h>
+
+#ifndef __INTEL_COMPILER
+#include <cmath> // for HUGE_VAL
+#endif
+
 // This file specifies some platform dependant functions, regarding the FPU
 // directed rounding modes.  There is only support for double precision.
 //
 // It also contains the definition of the Protect_FPU_rounding<> class,
 // which helps to protect blocks of code needing a particular rounding mode.
 
-#if defined __alpha__  && defined __linux__ 
+#if defined __alpha__  && defined __linux__
 extern "C" {
 #  include <fenv.h>
 }
-#elif defined __linux__ && !defined __PGI && !defined __ia64__ \
-  && !defined __x86_64__
+#elif defined __powerpc__
 #  include <fpu_control.h>
-#elif defined __SUNPRO_CC || (defined __KCC && defined __sun)
+#elif defined __SUNPRO_CC && defined __sun
 #  include <ieeefp.h>
-#elif defined __osf || defined __osf__ 
+#elif defined __osf || defined __osf__
 #  ifdef __GNUG__
      // GCC seems to remove (fixincludes) read_rnd/write_rnd...
 #    include "/usr/include/float.h"
@@ -48,10 +53,8 @@ extern "C" {
 #  endif
 #elif defined __BORLANDC__
 #  include <float.h>
-#elif defined __sgi
-#  include <sys/fpu.h>
 #elif defined _MSC_VER || defined __sparc__ || \
-     (defined __i386__ && !defined __PGI)
+     (defined __i386__ && !defined __PGI && !defined __SUNPRO_CC)
    // Nothing to include.
 #else
    // By default we use the ISO C99 version.
@@ -59,14 +62,6 @@ extern "C" {
 #endif
 
 
-// GCC 3.0.0 has some bugs, which can be worked around, but it's
-// not worth maintaining them anymore.
-#if defined __GNUG__ && !defined __INTEL_COMPILER && \
-    (__GNUG__ == 3 && __GNUC_MINOR__ == 0 && __GNUC_PATCHLEVEL__ == 0)
-#  error GCC 3.0.0 is buggy, use at your own risk.
-#endif
-
-
 // Some useful constants
 
 #if defined CGAL_CFG_NO_LIMITS
@@ -81,7 +76,7 @@ extern "C" {
 #else
 #  include <limits>
 #  define CGAL_IA_MIN_DOUBLE std::numeric_limits<double>::denorm_min()
-#  define CGAL_IA_MAX_DOUBLE std::numeric_limits<double>::max()
+#  define CGAL_IA_MAX_DOUBLE (std::numeric_limits<double>::max)()
 #endif
 
 
@@ -148,26 +143,16 @@ inline double IA_force_to_double(double x)
 
 // We sometimes need to stop constant propagation,
 // because operations are done with a wrong rounding mode at compile time.
-// G++ also uses __builtin_constant_p().
 #ifndef CGAL_IA_DONT_STOP_CONSTANT_PROPAGATION
-#  if defined __GNUG__ && __GNUG__ < 3
-	// Note : GCC 3 doesn't guarantee __builtin_constant_p to return false
-	// when he will not do cprop :(.
-#    define CGAL_IA_STOP_CPROP(x) \
-            (__builtin_constant_p (x) ? CGAL::IA_force_to_double(x) : (x) )
-#    define CGAL_IA_STOP_CPROP2(x,y) \
-            (__builtin_constant_p (y) ? CGAL_IA_STOP_CPROP(x) : (x) )
-#  else
-#    define CGAL_IA_STOP_CPROP(x)    CGAL::IA_force_to_double(x)
-#    define CGAL_IA_STOP_CPROP2(x,y) CGAL::IA_force_to_double(x)
-#  endif
+#  define CGAL_IA_STOP_CPROP(x)    CGAL::IA_force_to_double(x)
+#  define CGAL_IA_STOP_CPROP2(x,y) CGAL::IA_force_to_double(x)
 #else
 #  define CGAL_IA_STOP_CPROP(x)    (x)
 #  define CGAL_IA_STOP_CPROP2(x,y) (x)
 #endif
 
 // std::sqrt(double) on VC++ and CygWin is buggy when not optimizing.
-#ifdef _MSC_VER
+#if defined ( _MSC_VER ) && ! defined ( _WIN64 )
 inline double IA_bug_sqrt(double d)
 {
   _asm
@@ -179,7 +164,7 @@ inline double IA_bug_sqrt(double d)
   return d;
 }
 
-#  define CGAL_BUG_SQRT(d) CGAL::IA_bug_sqrt(d)
+#  define CGAL_BUG_SQRT(d) IA_bug_sqrt(d)
 
 
 #elif defined __SSE2_MATH__
@@ -192,7 +177,7 @@ inline double IA_bug_sqrt(double d)
   asm volatile ("fsqrt" : "=t"(r) : "0"(d));
   return r;
 }
-#  define CGAL_BUG_SQRT(d) CGAL::IA_bug_sqrt(d)
+#  define CGAL_BUG_SQRT(d) IA_bug_sqrt(d)
 #else
 #  define CGAL_BUG_SQRT(d) CGAL_CLIB_STD::sqrt(d)
 #endif
@@ -210,9 +195,9 @@ inline double IA_bug_sqrt(double d)
         CGAL_IA_FORCE_TO_DOUBLE(CGAL_BUG_SQRT(CGAL_IA_STOP_CPROP(a)))
 
 
-#if defined __i386__ && !defined __PGI
+#if defined __i386__ && !defined __PGI && !defined __SUNPRO_CC
 
-#  if defined CGAL_SAFE_SSE2 
+#  if defined CGAL_SAFE_SSE2
 
 #define CGAL_IA_SETFPCW(CW) _MM_SET_ROUNDING_MODE(CW)
 #define CGAL_IA_GETFPCW(CW) CW = _MM_GET_ROUNDING_MODE()
@@ -225,9 +210,10 @@ typedef unsigned int FPU_CW_t;
 #  else
 // The GNU libc version (cf powerpc) is nicer, but doesn't work on libc 5 :(
 // This one also works with CygWin.
-// Note that the ISO C99 version is not enough because of the extended
-// mantissa issue on x86 (required by Fixed_precision_nt, modular computations
-// in the future, but not IA right now).
+// Note that the ISO C99 version may not be enough because of the extended
+// mantissa issue on x86 (may be required by some kinds of computation, but
+// as far as CGAL::Interval_nt<> is concerned, the double-rounding issues
+// are taking care of there).
 #define CGAL_IA_SETFPCW(CW) asm volatile ("fldcw %0" : :"m" (CW))
 #define CGAL_IA_GETFPCW(CW) asm volatile ("fnstcw %0" : "=m" (CW))
 typedef unsigned short FPU_CW_t;
@@ -238,7 +224,7 @@ typedef unsigned short FPU_CW_t;
 
 #  endif
 
-#elif defined __powerpc__  
+#elif defined __powerpc__
 #define CGAL_IA_SETFPCW(CW) _FPU_SETCW(CW)
 #define CGAL_IA_GETFPCW(CW) _FPU_GETCW(CW)
 typedef fpu_control_t FPU_CW_t;
@@ -247,7 +233,7 @@ typedef fpu_control_t FPU_CW_t;
 #define CGAL_FE_UPWARD       (_FPU_RC_UP      | _FPU_DEFAULT)
 #define CGAL_FE_DOWNWARD     (_FPU_RC_DOWN    | _FPU_DEFAULT)
 
-#elif defined __SUNPRO_CC || (defined __KCC && defined __sun)
+#elif defined __SUNPRO_CC && defined __sun
 #define CGAL_IA_SETFPCW(CW) fpsetround(fp_rnd(CW))
 #define CGAL_IA_GETFPCW(CW) CW = fpgetround()
 typedef unsigned int FPU_CW_t;
@@ -265,33 +251,7 @@ typedef unsigned int FPU_CW_t;
 #define CGAL_FE_UPWARD       (0x80000000 | 0x20000000 | 0x1f)
 #define CGAL_FE_DOWNWARD     (0xc0000000 | 0x20000000 | 0x1f)
 
-#elif defined __sgi
-typedef unsigned int FPU_CW_t;
-
-inline FPU_CW_t sgi_get_fpu_cw()
-{
-  fpc_csr csr;
-  csr.fc_word = get_fpc_csr();
-  return csr.fc_struct.rounding_mode;
-}
-
-inline void sgi_set_fpu_cw(FPU_CW_t cw)
-{
-  fpc_csr csr;
-  csr.fc_word = get_fpc_csr();
-  csr.fc_struct.rounding_mode = cw;
-  csr.fc_struct.flush = 0; // By default, denormals are flushed to zero !
-  set_fpc_csr(csr.fc_word);
-}
-
-#define CGAL_IA_SETFPCW(CW)  sgi_set_fpu_cw(CW)
-#define CGAL_IA_GETFPCW(CW)  CW = sgi_get_fpu_cw()
-#define CGAL_FE_TONEAREST    ROUND_TO_NEAREST
-#define CGAL_FE_TOWARDZERO   ROUND_TO_ZERO
-#define CGAL_FE_UPWARD       ROUND_TO_PLUS_INFINITY
-#define CGAL_FE_DOWNWARD     ROUND_TO_MINUS_INFINITY
-
-#elif defined __mips__ // && !defined __sgi
+#elif defined __mips__
 #define CGAL_IA_SETFPCW(CW) asm volatile ("ctc1 %0,$31" : :"r" (CW))
 #define CGAL_IA_GETFPCW(CW) asm volatile ("cfc1 %0,$31" : "=r" (CW))
 typedef unsigned int FPU_CW_t;
@@ -314,14 +274,21 @@ typedef unsigned int FPU_CW_t;
 #define CGAL_IA_GETFPCW(CW) (CW = __ieee_get_fp_control())
 typedef unsigned long FPU_CW_t;
 #define CGAL_FE_TONEAREST   FE_TONEAREST
-#define CGAL_FE_TOWARDZERO  FE_TOWARDZERO 
-#define CGAL_FE_UPWARD      FE_UPWARD 
-#define CGAL_FE_DOWNWARD    FE_DOWNWARD 
+#define CGAL_FE_TOWARDZERO  FE_TOWARDZERO
+#define CGAL_FE_UPWARD      FE_UPWARD
+#define CGAL_FE_DOWNWARD    FE_DOWNWARD
 
-#elif defined ( _MSC_VER ) 
+#elif defined ( _MSC_VER )
+#if ( _MSC_VER < 1400)
 #define CGAL_IA_SETFPCW(CW) _controlfp (CW, _MCW_RC )
 #define CGAL_IA_GETFPCW(CW) CW = _controlfp (0, 0 ) &  _MCW_RC
 typedef unsigned short FPU_CW_t;
+#else
+#define CGAL_IA_SETFPCW(CW) unsigned int dummy; _controlfp_s (&dummy, CW, _MCW_RC )
+#define CGAL_IA_GETFPCW(CW)_controlfp_s (&CW, 0, 0 ); CW  &=  _MCW_RC
+typedef unsigned int FPU_CW_t;
+#endif
+
 #define CGAL_FE_TONEAREST    _RC_NEAR
 #define CGAL_FE_TOWARDZERO   _RC_CHOP
 #define CGAL_FE_UPWARD       _RC_UP
@@ -340,7 +307,7 @@ typedef unsigned short FPU_CW_t;
 // This is a version following the ISO C99 standard, which aims at portability.
 // The drawbacks are speed on one hand, and also, on x86, it doesn't fix the
 // extended mantissa issue (this is not a problem for IA, but it is one for
-// Fixed_precision_nt, and some future modular computations as well).
+// some future modular computations).
 #define CGAL_IA_SETFPCW(CW)  fesetround(CW)
 #define CGAL_IA_GETFPCW(CW)  CW = fegetround()
 typedef int FPU_CW_t;
@@ -379,7 +346,6 @@ FPU_get_and_set_cw (FPU_CW_t cw)
 
 
 // The following is meant to truncate the mantissa of x86 FPUs to 53 bits.
-// It is used by the Fixed_precision_nt.
 inline void force_ieee_double_precision()
 {
 #if defined __i386__ || defined _MSC_VER || defined __BORLANDC__
@@ -391,7 +357,7 @@ inline void force_ieee_double_precision()
 // and whose destructor resets it back to the saved state.
 
 template <bool Protected = true> struct Protect_FPU_rounding;
- 
+
 template <>
 struct Protect_FPU_rounding<true>
 {
@@ -406,7 +372,7 @@ struct Protect_FPU_rounding<true>
 private:
   FPU_CW_t backup;
 };
- 
+
 template <>
 struct Protect_FPU_rounding<false>
 {
