@@ -61,7 +61,8 @@ bool projected_vertex_cycle_to_nef_3 (typename Nef_3::SNC_structure &snc,
    typedef CGAL::SNC_point_locator_by_spatial_subdivision
            <CGAL::SNC_decorator<SNC_structure> >    Point_locator;
 
-   typedef CGAL::SNC_constructor<SNC_structure>     SNC_constructor;
+   typedef typename SNC_structure::Items               Items;
+   typedef CGAL::SNC_constructor<Items, SNC_structure> SNC_constructor;
 
    // declarations and defaults
    II v_it, v_pred_it;
@@ -117,7 +118,7 @@ bool projected_vertex_cycle_to_nef_3 (typename Nef_3::SNC_structure &snc,
    CTP_vertex_handle t_vh, t_vh_0;
    CTP_face_handle t_fh;
    CTP_face_circulator t_fc, t_fc_0;
-   int idx;
+   int idx(0);
    {  // search a constrained edge from "outside"
       t_fh = ctp.infinite_face ();
       cond = t_fh->has_vertex(ctp.infinite_vertex(), idx);
@@ -128,7 +129,7 @@ bool projected_vertex_cycle_to_nef_3 (typename Nef_3::SNC_structure &snc,
 
       t_fc = t_fc_0 = ctp.incident_faces(t_vh, t_fh);
       do
-      {  if (cond=t_fc->is_constrained(ctp.cw(t_fc->index(t_vh)))) break;
+      {  if ((cond=t_fc->is_constrained(ctp.cw(t_fc->index(t_vh))))) break;
       } while (--t_fc != t_fc_0)
       ; CGAL_assertion ( cond ); // do-while ends with break
       t_fh = t_fc->neighbor(ctp.cw(t_fc->index(t_vh)));
@@ -189,7 +190,7 @@ bool projected_vertex_cycle_to_nef_3 (typename Nef_3::SNC_structure &snc,
          p_seh->mark() = p_seh->twin()->mark() = true;
 
 	 // constrained edge detected?
-	 if ( cond = t_fc->is_constrained(ctp.cw(idx)) )
+	 if ((cond = t_fc->is_constrained(ctp.cw(idx))))
 	 {  // create new sface in SM
 	    SFace_handle p_fh = p_dec.new_sface ();
 	    p_fh->mark() = false;
