@@ -15,8 +15,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.2-branch/Optimisation_basic/include/CGAL/Optimisation/Access_coordinates_begin_d.h $
-// $Id: Access_coordinates_begin_d.h 28567 2006-02-16 14:30:13Z lsaboret $
+// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.3-branch/Optimisation_basic/include/CGAL/Optimisation/Access_coordinates_begin_d.h $
+// $Id: Access_coordinates_begin_d.h 36740 2007-03-01 16:52:45Z gaertner $
 // 
 //
 // Author(s)     : Sven Schoenherr <sven@inf.ethz.ch>
@@ -47,7 +47,7 @@ class Access_coordinates_begin_d {
 
     // types
     typedef  typename R::Point_d        Point;
-    typedef  const typename R::RT *     Coordinate_iterator;
+    typedef  typename Point::Homogeneous_const_iterator Coordinate_iterator;
 
     // unary function class types
     typedef  Coordinate_iterator        result_type;
@@ -70,13 +70,7 @@ private:
 public:
     Coordinate_iterator
     operator() ( const Point& p) const { 
-      typename R::Rep_tag tag;
-#if defined(__sun) && defined(__SUNPRO_CC)
-    // to avoid a warning "tag has not yet been assigned a value"
-    typedef typename R::Rep_tag Rep_tag;
-    tag = Rep_tag();
-#endif // SUNPRO
-      return access(p, tag);
+      return p.homogeneous_begin();
     }
 };
 
