@@ -12,14 +12,11 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.2-branch/Kinetic_data_structures/include/CGAL/Polynomial/internal/config.h $
-// $Id: config.h 28655 2006-02-21 05:49:40Z drussel $
+// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.3-branch/Kinetic_data_structures/include/CGAL/Polynomial/internal/config.h $
+// $Id: config.h 35766 2007-01-20 21:39:01Z drussel $
 // 
 //
 // Author(s)     : Daniel Russel <drussel@alumni.princeton.edu>
 
 #define CGAL_POLYNOMIAL_USE_CGAL
-#ifdef CGAL_USE_CORE
-#define CGAL_POLYNOMIAL_USE_CORE
-#endif
 //#define POLYNOMIAL_USE_GSL
