@@ -11,8 +11,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.2-branch/Stream_lines_2/include/CGAL/Stream_lines_2.h $
-// $Id: Stream_lines_2.h 30667 2006-04-19 16:56:12Z glisse $
+// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.3-branch/Stream_lines_2/include/CGAL/Stream_lines_2.h $
+// $Id: Stream_lines_2.h 38262 2007-04-18 08:54:16Z spion $
 // 
 //
 // Author(s)     : Abdelkrim Mebarki <Abdelkrim.Mebarki@sophia.inria.fr>
@@ -85,9 +85,9 @@ protected:
   {
   public:
     bool operator()(const Pq_element &a1, const Pq_element
-		    &a2)
+        &a2)
       {
-	return a1.fourth.second < a2.fourth.second ;
+  return a1.fourth.second < a2.fourth.second ;
       }
   };
   std::priority_queue<Pq_element, std::vector<Pq_element>, C>    pq;
@@ -118,45 +118,45 @@ public:
       pPoint = Point_2(max_x+separating_distance,max_y+separating_distance);
       m_DT.insert(pPoint);
       for (int i=(int) (min_x-separating_distance);i<max_x+(int)
-	     separating_distance;i=i+(int) (fSepStl_seed))
-	{
-	  pPoint = Point_2((FT)i,(FT)max_y+separating_distance);
-	  m_DT.insert(pPoint);
-	  pPoint = Point_2((FT)max_x+separating_distance,(FT)i);
-	  m_DT.insert(pPoint);
-	  pPoint = Point_2((FT)i,min_y-separating_distance);
-	  m_DT.insert(pPoint);
-	  pPoint = Point_2(min_x-separating_distance,(FT)i);
-	  m_DT.insert(pPoint);
-	}
+       separating_distance;i=i+(int) (fSepStl_seed))
+  {
+    pPoint = Point_2((FT)i,(FT)max_y+separating_distance);
+    m_DT.insert(pPoint);
+    pPoint = Point_2((FT)max_x+separating_distance,(FT)i);
+    m_DT.insert(pPoint);
+    pPoint = Point_2((FT)i,min_y-separating_distance);
+    m_DT.insert(pPoint);
+    pPoint = Point_2(min_x-separating_distance,(FT)i);
+    m_DT.insert(pPoint);
+  }
       _number_of_lines = 0;
       place_stream_lines(*vf_2, *int_2,
-			 samp_step);     
+       samp_step);     
     }
 protected:
   void place_stream_lines(const Vector_field_2 & vector_field_2, const Integrator_2 & integrator,
-			  const int & sampling_step, const bool & step_by_step = false);
-  bool get_next_seed_point(FT & distanceg, Point_2 & seed_point);
+        const int & sampling_step, const bool & step_by_step = false);
+  bool get_next_seed_point(FT & distanceg, Point_2 & seed_point_);
   FT find_smallest_circle(const Vertex_handle & pVertex_handle);
   Vertex_handle insert_point(const Point_2 & pPoint, FT& fDist,bool bDistanceCalculation);
   Vertex_handle insert_point(const Point_2 & pPoint, const Face_handle & m_Face_handle, FT& fDist,
-			     bool bDistanceCalculation);
-  void integrate_streamline(const Vector_field_2 & vector_field_2,	const Integrator_2 & integrator,
-			    Point_container_2& stl, Point_2& seed_point, Vertex_container_2& stl_vertices, const int & sampling_step);
-  void integrate_forward(const Vector_field_2 & vector_field_2,	const Integrator_2 & integrator,
-			 Point_container_2& stl,Point_2&	seed_point,
-			 Vertex_container_2& stl_vertices, const int & sampling_step);
-  void integrate_backward(const Vector_field_2 & vector_field_2,	const Integrator_2 & integrator, Point_container_2& 
-			  stl, Vertex_container_2& stl_vertices, const int & sampling_step);
+           bool bDistanceCalculation);
+  void integrate_streamline(const Vector_field_2 & vector_field_2,  const Integrator_2 & integrator,
+          Point_container_2& stl, Point_2& seed_point_, Vertex_container_2& stl_vertices, const int & sampling_step);
+  void integrate_forward(const Vector_field_2 & vector_field_2, const Integrator_2 & integrator,
+       Point_container_2& stl,Point_2&  seed_point,
+       Vertex_container_2& stl_vertices, const int & sampling_step);
+  void integrate_backward(const Vector_field_2 & vector_field_2,  const Integrator_2 & integrator, Point_container_2& 
+        stl, Vertex_container_2& stl_vertices, const int & sampling_step);
   void insert_streamline(const Vector_field_2 & vector_field_2, Point_container_2 stl,
-			 Vertex_container_2 stl_vertices);
+       Vertex_container_2 stl_vertices);
   void pq_elements(const Vector_field_2 & vector_field_2, Vertex_container_2 stl_vertices, int i,
-		   const Vertex_handle & m_Vertex_handle, int before_end);
+       const Vertex_handle & m_Vertex_handle, int before_end);
   void make_iterator();
 public:
   Stream_lines_2(const Vector_field_2 & m_vector_field_2, const Integrator_2 & m_integrator, const FT
-		 & m_separating_distance, const FT & m_saturation_ratio, const int & sampling_insertion = 0, const bool & 
-		 step_by_step = false);
+     & m_separating_distance, const FT & m_saturation_ratio, const int & sampling_insertion = 0, const bool & 
+     step_by_step = false);
   bool continue_next(const Vector_field_2 & vector_field_2, const Integrator_2 & integrator, const int & sampling_step);
   Stream_line_iterator_2 begin();
   Stream_line_iterator_2 end();
@@ -170,13 +170,13 @@ public:
       std::list< std::pair<Point_2, Point_2> > _list;
       Edge_iterator eit = m_DT.edges_begin();
       for (;eit != m_DT.edges_end();eit++)
-	{
-	  Point_2 p1 = (*eit).first->vertex(m_DT.ccw((*eit).second))->point();
-	  Point_2 p2 = (*eit).first->vertex(m_DT.cw((*eit).second))->point();
-	  _list.push_front(std::pair<Point_2, Point_2>(p1, p2));
-	}
+  {
+    Point_2 p1 = (*eit).first->vertex(m_DT.ccw((*eit).second))->point();
+    Point_2 p2 = (*eit).first->vertex(m_DT.cw((*eit).second))->point();
+    _list.push_front(std::pair<Point_2, Point_2>(p1, p2));
+  }
       return _list;
-    }	
+    } 
   std::pair<Point_2, FT> get_biggest_circle()
     {
       Pq_element m_Pq = Biggest_circle;
@@ -190,7 +190,7 @@ protected:
   FT min_y;
 protected:
   Stream_line_iterator_2 begin_iterator;
-  Stream_line_iterator_2 end_iterator;	
+  Stream_line_iterator_2 end_iterator;  
 private:
   int number_of_points;
 };
@@ -199,7 +199,7 @@ template <class VectorField_2, class Integrator_2>
 Stream_lines_2<VectorField_2, Integrator_2>::Stream_lines_2(const Vector_field_2 &
 vector_field_2, const Integrator_2 & m_integrator, const FT & m_separating_distance, const FT
 & m_saturation_ratio, const int & sampling_step, const bool &
-							    step_by_step)
+                  step_by_step)
 {
   separating_distance = m_separating_distance;
   saturation_ratio = m_saturation_ratio;
@@ -220,7 +220,7 @@ vector_field_2, const Integrator_2 & m_integrator, const FT & m_separating_dista
   pPoint = Point_2(max_x+separating_distance,max_y+separating_distance);
   m_DT.insert(pPoint);
   for (int i=(int) (min_x-separating_distance);i<max_x+(int)
-	 separating_distance;i=i+(int) (fSepStl_seed))
+   separating_distance;i=i+(int) (fSepStl_seed))
     {
       pPoint = Point_2((FT)i,(FT)max_y+separating_distance);
       m_DT.insert(pPoint);
@@ -237,11 +237,11 @@ vector_field_2, const Integrator_2 & m_integrator, const FT & m_separating_dista
   samp_step = sampling_step;
   stl_container.clear();
   place_stream_lines(vector_field_2, m_integrator,
-		     sampling_step, step_by_step);
+         sampling_step, step_by_step);
 }
 
 template <class VectorField_2, class Integrator_2>
-void Stream_lines_2<VectorField_2, Integrator_2>::place_stream_lines(const Vector_field_2 & vector_field_2,	const Integrator_2 & integrator, const int & sampling_step, const bool & step_by_step)
+void Stream_lines_2<VectorField_2, Integrator_2>::place_stream_lines(const Vector_field_2 & vector_field_2, const Integrator_2 & integrator, const int & sampling_step, const bool & step_by_step)
 {
   seed_point = Point_2((max_x+min_x)/2.0,(max_y+min_y)/2.0);
   // the first chosen point can be not valid
@@ -249,25 +249,25 @@ void Stream_lines_2<VectorField_2, Integrator_2>::place_stream_lines(const Vecto
   FT yrange = max_y - min_y;
   while(!vector_field_2.is_in_domain(seed_point))
     {
-      std::cout << "searching valid seed point..\n";
+//       std::cout << "searching valid seed point..\n";
       FT x = min_x + (FT) (((FT) std::rand() * xrange)/((FT) RAND_MAX));
       FT y = min_y + (FT) (((FT) std::rand() * yrange)/((FT) RAND_MAX));
       seed_point = Point_2(x, y);
     }
-  std::cout << seed_point << " first seed point\n";
-  std::cout << "creating the placement..\n";
+//   std::cout << seed_point << " first seed point\n";
+//   std::cout << "creating the placement..\n";
   FT distance = (FT) max_x * (1.0/2.0);
   bool b = (distance>fSepStl_seed);
-  // 	int i=0;
+  //  int i=0;
   if (!step_by_step)
     while(b)
       {
-	Point_container_2 stl;
-	Vertex_container_2 stl_vertices;
-	integrate_streamline(vector_field_2, integrator, stl, seed_point, stl_vertices, sampling_step);
-	insert_streamline(vector_field_2, stl, stl_vertices);
-	_number_of_lines++;
-	b = get_next_seed_point(distance,seed_point);
+  Point_container_2 stl;
+  Vertex_container_2 stl_vertices;
+  integrate_streamline(vector_field_2, integrator, stl, seed_point, stl_vertices, sampling_step);
+  insert_streamline(vector_field_2, stl, stl_vertices);
+  _number_of_lines++;
+  b = get_next_seed_point(distance,seed_point);
       }
   else
     {
@@ -280,7 +280,7 @@ void Stream_lines_2<VectorField_2, Integrator_2>::place_stream_lines(const Vecto
     }
   make_iterator();
 }
-	
+  
 template <class VectorField_2, class Integrator_2>
 bool Stream_lines_2<VectorField_2, Integrator_2>::continue_next(const Vector_field_2 & vector_field_2, const Integrator_2 & integrator, const int & sampling_step)
 {
@@ -298,10 +298,10 @@ bool Stream_lines_2<VectorField_2, Integrator_2>::continue_next(const Vector_fie
 template <class VectorField_2, class Integrator_2>
 void 
 Stream_lines_2<VectorField_2,Integrator_2>::integrate_streamline(const Vector_field_2 & vector_field_2, const Integrator_2 & integrator, Point_container_2& stl, Point_2&
-seed_point, Vertex_container_2& stl_vertices, const int & sampling_step)
+seed_point_, Vertex_container_2& stl_vertices, const int & sampling_step)
 {
-	integrate_forward(vector_field_2, integrator, stl, seed_point, stl_vertices, sampling_step);
-	integrate_backward(vector_field_2, integrator, stl, stl_vertices, sampling_step);
+  integrate_forward(vector_field_2, integrator, stl, seed_point_, stl_vertices, sampling_step);
+  integrate_backward(vector_field_2, integrator, stl, stl_vertices, sampling_step);
 }
 
 template <class VectorField_2, class Integrator_2>
@@ -315,15 +315,15 @@ Stream_lines_2<VectorField_2, Integrator_2>::find_smallest_circle(const Vertex_h
   CGAL_For_all(pFace_handle,pEnd)
     {
       FT fDist =
-	CGAL::squared_radius(
-			     pFace_handle->vertex(0)->point(),
-			     pFace_handle->vertex(1)->point(),
-			     pFace_handle->vertex(2)->point()) * 4.0;
+  CGAL::squared_radius(
+           pFace_handle->vertex(0)->point(),
+           pFace_handle->vertex(1)->point(),
+           pFace_handle->vertex(2)->point()) * 4.0;
       fDist = sqrt(fDist);
       if (fDist < fMin)
-	{
-	  fMin = fDist;
-	}
+  {
+    fMin = fDist;
+  }
     }
   return fMin;
 }
@@ -356,7 +356,7 @@ Stream_lines_2<VectorField_2,Integrator_2>::insert_point(const Point_2 & pPoint,
 
 template <class VectorField_2, class Integrator_2>
 void
-Stream_lines_2<VectorField_2, Integrator_2>::integrate_forward(const Vector_field_2 & vector_field_2, const Integrator_2 & integrator, Point_container_2& stl, Point_2& seed_point, Vertex_container_2& stl_vertices, const int & sampling_step)
+Stream_lines_2<VectorField_2, Integrator_2>::integrate_forward(const Vector_field_2 & vector_field_2, const Integrator_2 & integrator, Point_container_2& stl, Point_2& seed_point_, Vertex_container_2& stl_vertices, const int & sampling_step)
 {
   int sampling = 0; // sampling step;
   int insertion = 0; // insertion order;
@@ -367,13 +367,13 @@ Stream_lines_2<VectorField_2, Integrator_2>::integrate_forward(const Vector_fiel
   Point_2 pPoint1;
   bool bEnd = false;
   FT dist;
-  Point_2 new_point = Point_2 (seed_point.x(), seed_point.y());
+  Point_2 new_point = Point_2 (seed_point_.x(), seed_point_.y());
   Vertex_handle m_Vertex_handle = insert_point(new_point, dist, true);
   stl_vertices.push_front(m_Vertex_handle);
   stl.push_front(new_point);
   number_of_points++;
-  Point_2 old_point;
-  insertion_step = (int) (((dist)-fSepStl_seed) / std::max((FT) sampling_step,vector_field_2.get_integration_step()));
+  Point_2 old_point = seed_point_;
+  insertion_step = (int) (((dist)-fSepStl_seed) / (std::max)((FT) sampling_step,vector_field_2.get_integration_step()));
   if (insertion_step < 0) insertion_step = 0;
   while (!bEnd)
     {
@@ -384,103 +384,103 @@ Stream_lines_2<VectorField_2, Integrator_2>::integrate_forward(const Vector_fiel
       bEnd = !vector_field_2.is_in_domain(new_point);
       bEnd = bEnd || (new_point == old_point);/* to review */
       if(number_of_points > 30)
-	bEnd = bEnd || ((distance(stl.front(), stl.back()))<vector_field_2.get_integration_step());
+  bEnd = bEnd || ((distance(stl.front(), stl.back()))<vector_field_2.get_integration_step());
       FT dist_ = distance(ex_old_point,new_point);
       bEnd = bEnd || dist_ <= vector_field_2.get_integration_step();
       if (!bEnd)
-	{
-	  if(sampling != sampling_step)
-	    {
-	      stl.push_front(new_point);
-	      number_of_points ++;
-	      sampling++;
-	    }
-	  else
-	    {
-	      if (insertion != insertion_step)
-		{
-		  stl.push_front(new_point); 
-		  number_of_points++;
-		  insertion++;
-		  list_of_point.push_front(new_point);
-		}
-	      else
-		{
-		  stl.push_front(new_point);
-		  number_of_points++;
-		  list_of_point.push_front(new_point);
-		  list_of_point.pop_front();
-		  m_Vertex_handle = insert_point(new_point, stl_vertices.front()->face(), dist, true);
-		  while ((dist <= separating_distance)&&(!list_of_point.empty()))
-		    {
-		      m_DT.remove(m_Vertex_handle);
-		      for (int i=0;i<=sampling_step;i++){
-			stl.pop_front();
-			number_of_points--;}
-		      new_point = list_of_point.front();
-		      list_of_point.pop_front();
-		      m_Vertex_handle = insert_point(new_point, stl_vertices.front()->face(), dist,true);
-		    }
-		  // adaptive insertion order coefficient
-		  insertion_step = (int) (((dist)-fSepStl_seed) /
-					  std::max((FT) sampling_step,vector_field_2.get_integration_step()));
-		  if (insertion_step < 0) insertion_step = 0;
-		  list_of_vertex.push_front(m_Vertex_handle);
-		  (bEnd) = (((bEnd))||(dist<separating_distance));
-		  while (!list_of_point.empty())
-		    {
-		      Point_2 p = list_of_point.front();
-		      m_Vertex_handle = insert_point(p, stl_vertices.front()->face(), dist, false);
-		      list_of_vertex.push_front(m_Vertex_handle);
-		      list_of_point.pop_front();
-		    }
-		  while(!list_of_vertex.empty())
-		    {
-		      stl_vertices.push_front(list_of_vertex.front());
-		      list_of_vertex.pop_front();
-		    }
-		  insertion = 0;
-		}
-	      sampling = 0;
-	    }
-	}
+  {
+    if(sampling != sampling_step)
+      {
+        stl.push_front(new_point);
+        number_of_points ++;
+        sampling++;
+      }
+    else
+      {
+        if (insertion != insertion_step)
+    {
+      stl.push_front(new_point); 
+      number_of_points++;
+      insertion++;
+      list_of_point.push_front(new_point);
+    }
+        else
+    {
+      stl.push_front(new_point);
+      number_of_points++;
+      list_of_point.push_front(new_point);
+      list_of_point.pop_front();
+      m_Vertex_handle = insert_point(new_point, stl_vertices.front()->face(), dist, true);
+      while ((dist <= separating_distance)&&(!list_of_point.empty()))
+        {
+          m_DT.remove(m_Vertex_handle);
+          for (int i=0;i<=sampling_step;i++){
+      stl.pop_front();
+      number_of_points--;}
+          new_point = list_of_point.front();
+          list_of_point.pop_front();
+          m_Vertex_handle = insert_point(new_point, stl_vertices.front()->face(), dist,true);
+        }
+      // adaptive insertion order coefficient
+      insertion_step = (int) (((dist)-fSepStl_seed) /
+            (std::max)((FT) sampling_step,vector_field_2.get_integration_step()));
+      if (insertion_step < 0) insertion_step = 0;
+      list_of_vertex.push_front(m_Vertex_handle);
+      (bEnd) = (((bEnd))||(dist<separating_distance));
+      while (!list_of_point.empty())
+        {
+          Point_2 p = list_of_point.front();
+          m_Vertex_handle = insert_point(p, stl_vertices.front()->face(), dist, false);
+          list_of_vertex.push_front(m_Vertex_handle);
+          list_of_point.pop_front();
+        }
+      while(!list_of_vertex.empty())
+        {
+          stl_vertices.push_front(list_of_vertex.front());
+          list_of_vertex.pop_front();
+        }
+      insertion = 0;
+    }
+        sampling = 0;
+      }
+  }
       else
-	{
-	  if (!list_of_point.empty())
-	    {
-	      new_point = list_of_point.front();
-	      list_of_point.pop_front();
-	      Vertex_handle m_Vertex_handle = insert_point(new_point, dist, true);
-	      while ((dist <= separating_distance)&&(!list_of_point.empty()))
-		{
-		  m_DT.remove(m_Vertex_handle);
-		  for (int i=0;i<=sampling_step;i++)
-		    {
-		      stl.pop_front();
-		      number_of_points--;
-		    }
-		  new_point = list_of_point.front();
-		  list_of_point.pop_front();
-		  m_Vertex_handle = insert_point(new_point, stl_vertices.front()->face(), dist, true);
-		}
-	      insertion_step =	(int) (((dist)-fSepStl_seed) / 
-				       std::max((FT) sampling_step,vector_field_2.get_integration_step()));
-	      if (insertion_step < 0) insertion_step = 0;
-	      (bEnd) = (((bEnd))||(dist<separating_distance));
-	    }
-	  while (!list_of_point.empty())
-	    {
-	      Point_2 p = list_of_point.front();
-	      m_Vertex_handle = insert_point(p, stl_vertices.front()->face(), dist, false);
-	      list_of_vertex.push_front(m_Vertex_handle);
-	      list_of_point.pop_front();
-	    }
-	  while(!list_of_vertex.empty())
-	    {
-	      stl_vertices.push_front(list_of_vertex.front());
-	      list_of_vertex.pop_front();
-	    }
-	}
+  {
+    if (!list_of_point.empty())
+      {
+        new_point = list_of_point.front();
+        list_of_point.pop_front();
+        Vertex_handle m_Vertex_handle = insert_point(new_point, dist, true);
+        while ((dist <= separating_distance)&&(!list_of_point.empty()))
+    {
+      m_DT.remove(m_Vertex_handle);
+      for (int i=0;i<=sampling_step;i++)
+        {
+          stl.pop_front();
+          number_of_points--;
+        }
+      new_point = list_of_point.front();
+      list_of_point.pop_front();
+      m_Vertex_handle = insert_point(new_point, stl_vertices.front()->face(), dist, true);
+    }
+        insertion_step =  (int) (((dist)-fSepStl_seed) / 
+               (std::max)((FT) sampling_step,vector_field_2.get_integration_step()));
+        if (insertion_step < 0) insertion_step = 0;
+        (bEnd) = (((bEnd))||(dist<separating_distance));
+      }
+    while (!list_of_point.empty())
+      {
+        Point_2 p = list_of_point.front();
+        m_Vertex_handle = insert_point(p, stl_vertices.front()->face(), dist, false);
+        list_of_vertex.push_front(m_Vertex_handle);
+        list_of_point.pop_front();
+      }
+    while(!list_of_vertex.empty())
+      {
+        stl_vertices.push_front(list_of_vertex.front());
+        list_of_vertex.pop_front();
+      }
+  }
     }
 }
 
@@ -502,7 +502,7 @@ void Stream_lines_2<VectorField_2, Integrator_2>::integrate_backward(const Vecto
   stl_vertices.push_back(m_Vertex_handle);
   stl.push_back(new_point);
   number_of_points++;
-  Point_2 old_point;
+  Point_2 old_point = new_point;
   while (!bEnd)
     {
       Point_2 ex_old_point = old_point;
@@ -512,108 +512,108 @@ void Stream_lines_2<VectorField_2, Integrator_2>::integrate_backward(const Vecto
       new_point = integrator(old_point,vector_field_2,false);
       bEnd = !vector_field_2.is_in_domain(new_point);
       FT dist_ = distance(ex_old_point,new_point);
-      bEnd = bEnd || dist_ <= vector_field_2.get_integration_step() || (new_point == old_point);/* to review */	
+      bEnd = bEnd || dist_ <= vector_field_2.get_integration_step() || (new_point == old_point);/* to review */ 
       if(number_of_points > 30)
-	bEnd = bEnd || ((distance(stl.front(), stl.back()))<vector_field_2.get_integration_step());
-      // 		bEnd = bEnd || (number_of_points > 3000);
+  bEnd = bEnd || ((distance(stl.front(), stl.back()))<vector_field_2.get_integration_step());
+      //    bEnd = bEnd || (number_of_points > 3000);
       if (!bEnd)
-	{
-	  if(sampling != sampling_step)
-	    {
-	      stl.push_back(new_point);
-	      number_of_points ++;
-	      sampling++;
-	    }
-	  else
-	    {
-	      if (insertion != insertion_step)
-		{
-		  stl.push_back(new_point); 
-		  number_of_points++;
-		  insertion++;
-		  list_of_point.push_back(new_point);
-		}
-	      else
-		{
-		  stl.push_back(new_point);
-		  number_of_points++;
-		  list_of_point.push_back(new_point);
-		  list_of_point.pop_back();
-		  m_Vertex_handle = insert_point(new_point, stl_vertices.back()->face(), dist,true);
-		  while ((dist <= separating_distance)&&(!list_of_point.empty()))
-		    {
-		      m_DT.remove(m_Vertex_handle);
-		      for (int i=0;i<=sampling_step;i++)
-			{
-			  stl.pop_back();
-			  number_of_points--;
-			}
-		      new_point = list_of_point.back();
-		      list_of_point.pop_back();
-		      m_Vertex_handle = insert_point(new_point, stl_vertices.back()->face(), dist,true);
-		    }
-		  // adaptive insertion order coefficient
-		  insertion_step =	(int) (((dist)-fSepStl_seed) / 
-					       std::max((FT) sampling_step,vector_field_2.get_integration_step()));
-		  if (insertion_step < 0) insertion_step = 0;
-		  list_of_vertex.push_back(m_Vertex_handle);
-		  (bEnd) = (((bEnd))||(dist<separating_distance));
-		  while (!list_of_point.empty())
-		    {
-		      Point_2 p = list_of_point.back();
-		      m_Vertex_handle = insert_point(p, stl_vertices.back()->face(), dist, false);
-		      list_of_vertex.push_front(m_Vertex_handle);
-		      list_of_point.pop_back();
-		    }
-		  while(!list_of_vertex.empty())
-		    {
-		      stl_vertices.push_back(list_of_vertex.back());
-		      list_of_vertex.pop_back();
-		    }
-		  insertion = 0;
-		}
-	      sampling = 0;
-	    }
-	}
+  {
+    if(sampling != sampling_step)
+      {
+        stl.push_back(new_point);
+        number_of_points ++;
+        sampling++;
+      }
+    else
+      {
+        if (insertion != insertion_step)
+    {
+      stl.push_back(new_point); 
+      number_of_points++;
+      insertion++;
+      list_of_point.push_back(new_point);
+    }
+        else
+    {
+      stl.push_back(new_point);
+      number_of_points++;
+      list_of_point.push_back(new_point);
+      list_of_point.pop_back();
+      m_Vertex_handle = insert_point(new_point, stl_vertices.back()->face(), dist,true);
+      while ((dist <= separating_distance)&&(!list_of_point.empty()))
+        {
+          m_DT.remove(m_Vertex_handle);
+          for (int i=0;i<=sampling_step;i++)
+      {
+        stl.pop_back();
+        number_of_points--;
+      }
+          new_point = list_of_point.back();
+          list_of_point.pop_back();
+          m_Vertex_handle = insert_point(new_point, stl_vertices.back()->face(), dist,true);
+        }
+      // adaptive insertion order coefficient
+      insertion_step =  (int) (((dist)-fSepStl_seed) / 
+                 (std::max)((FT) sampling_step,vector_field_2.get_integration_step()));
+      if (insertion_step < 0) insertion_step = 0;
+      list_of_vertex.push_back(m_Vertex_handle);
+      (bEnd) = (((bEnd))||(dist<separating_distance));
+      while (!list_of_point.empty())
+        {
+          Point_2 p = list_of_point.back();
+          m_Vertex_handle = insert_point(p, stl_vertices.back()->face(), dist, false);
+          list_of_vertex.push_front(m_Vertex_handle);
+          list_of_point.pop_back();
+        }
+      while(!list_of_vertex.empty())
+        {
+          stl_vertices.push_back(list_of_vertex.back());
+          list_of_vertex.pop_back();
+        }
+      insertion = 0;
+    }
+        sampling = 0;
+      }
+  }
       else
-	{
-	  if (!list_of_point.empty())
-	    {
-	      new_point = list_of_point.back();
-	      list_of_point.pop_back();
-	      Vertex_handle m_Vertex_handle = insert_point(new_point, stl_vertices.back()->face(), dist, true);
-	      while ((dist <= separating_distance)&&(!list_of_point.empty()))
-		{
-		  m_DT.remove(m_Vertex_handle);
-		  for (int i=0;i<=sampling_step;i++)
-		    {
-		      stl.pop_back();
-		      number_of_points--;
-		    }
-		  new_point = list_of_point.back();
-		  list_of_point.pop_back();
-		  m_Vertex_handle = insert_point(new_point, stl_vertices.back()->face(), dist, true);
-		}
-				// adaptive insertion order coefficient
-	      insertion_step =	(int) (((dist)-fSepStl_seed) / 
-				       std::max((FT) sampling_step,vector_field_2.get_integration_step()));
-	      if (insertion_step < 0) insertion_step = 0;
-	      // 				list_of_vertex.push_front(m_Vertex_handle);
-	      (bEnd) = (((bEnd))||(dist<separating_distance));
-	    }
-	  while (!list_of_point.empty())
-	    {
-	      Point_2 p = list_of_point.back();
-	      m_Vertex_handle = insert_point(p, stl_vertices.back()->face(), dist, false);
-	      list_of_vertex.push_back(m_Vertex_handle);
-	      list_of_point.pop_back();
-	    }
-	  while(!list_of_vertex.empty())
-	    {
-	      stl_vertices.push_back(list_of_vertex.back());
-	      list_of_vertex.pop_back();
-	    }
-	}
+  {
+    if (!list_of_point.empty())
+      {
+        new_point = list_of_point.back();
+        list_of_point.pop_back();
+        Vertex_handle m_Vertex_handle = insert_point(new_point, stl_vertices.back()->face(), dist, true);
+        while ((dist <= separating_distance)&&(!list_of_point.empty()))
+    {
+      m_DT.remove(m_Vertex_handle);
+      for (int i=0;i<=sampling_step;i++)
+        {
+          stl.pop_back();
+          number_of_points--;
+        }
+      new_point = list_of_point.back();
+      list_of_point.pop_back();
+      m_Vertex_handle = insert_point(new_point, stl_vertices.back()->face(), dist, true);
+    }
+        // adaptive insertion order coefficient
+        insertion_step =  (int) (((dist)-fSepStl_seed) / 
+               (std::max)((FT) sampling_step,vector_field_2.get_integration_step()));
+        if (insertion_step < 0) insertion_step = 0;
+        //        list_of_vertex.push_front(m_Vertex_handle);
+        (bEnd) = (((bEnd))||(dist<separating_distance));
+      }
+    while (!list_of_point.empty())
+      {
+        Point_2 p = list_of_point.back();
+        m_Vertex_handle = insert_point(p, stl_vertices.back()->face(), dist, false);
+        list_of_vertex.push_back(m_Vertex_handle);
+        list_of_point.pop_back();
+      }
+    while(!list_of_vertex.empty())
+      {
+        stl_vertices.push_back(list_of_vertex.back());
+        list_of_vertex.pop_back();
+      }
+  }
     }
 }
 
@@ -621,8 +621,8 @@ template <class VectorField_2, class Integrator_2>
 inline void
 Stream_lines_2<VectorField_2, Integrator_2>::
 insert_streamline(const Vector_field_2 & vector_field_2,
-		  Point_container_2 stl, Vertex_container_2
-		  stl_vertices)
+      Point_container_2 stl, Vertex_container_2
+      stl_vertices)
 {
   stl_container.push_back(stl);
   Vertex_handle m_Vertex_handle = NULL;
@@ -641,7 +641,7 @@ insert_streamline(const Vector_field_2 & vector_field_2,
 template <class VectorField_2, class Integrator_2>
 void Stream_lines_2<VectorField_2, Integrator_2>::
 pq_elements(const Vector_field_2 & vector_field_2, Vertex_container_2 stl_vertices, int i,
-		const Vertex_handle & m_Vertex_handle, int size_)
+    const Vertex_handle & m_Vertex_handle, int size_)
 {
   if ((i!=0) && (i!=(size_))){// && (std::div(i,10).rem!=0)){
     Vertex_handle pVertex_handle = stl_vertices.front();
@@ -650,87 +650,87 @@ pq_elements(const Vector_field_2 & vector_field_2, Vertex_container_2 stl_vertic
     if
       (m_DT.is_edge(pVertex_handle,m_Vertex_handle,pFace_handle,iIndex))
       {
-	Point_2 p0 = pVertex_handle->point();
-	Point_2 c = m_DT.circumcenter(pFace_handle);
-	FT fDist = distance(p0,c);
-	bool b = vector_field_2.is_in_domain(c) && (fDist >= fSepStl_seed);
-	if (b)
-	  {
-	    Circle pCircle(c,fDist);
-	    Pq_element m_Pq_element = Pq_element(
-						 pFace_handle->vertex(0),
-						 pFace_handle->vertex(1),
-						 pFace_handle->vertex(2),
-						 pCircle);
-	    if (ir == 0)
-	      {
-		Pq_previous_r = m_Pq_element;
-		Pq_element_max_r = m_Pq_element;
-		ir++;
-	      }
-	    else if (ir == 1)
-	      {
-		Pq_current_r =
-		  m_Pq_element;ir++;
-	      }
-	    else
-	      {
-		Pq_next_r = m_Pq_element;
-		if (Pq_element_max_r.fourth.second <= Pq_next_r.fourth.second)
-		  Pq_element_max_r = Pq_next_r;
-		if ((Pq_current_r.fourth.second>=Pq_previous_r.fourth.second)
-		    &&(Pq_current_r.fourth.second>=Pq_next_r.fourth.second))
-		  {
-		    pq.push(Pq_current_r);
-		  }
-		Pq_previous_r = Pq_current_r;
-		Pq_current_r = Pq_next_r;
-		ir++;
-	      }
-	  }
-	p0 = pFace_handle->neighbor(iIndex)->vertex(0)->point();
-	c = m_DT.circumcenter(pFace_handle->neighbor(iIndex));
-	fDist = distance(p0,c);
-	b = vector_field_2.is_in_domain(c) && (fDist >= fSepStl_seed);
-	if (b)
-	  {
-	    Circle pCircle(c,fDist);
-	    Pq_element m_Pq_element = Pq_element(
-						 pFace_handle->neighbor(iIndex)->vertex(0),
-						 pFace_handle->neighbor(iIndex)->vertex(1),
-						 pFace_handle->neighbor(iIndex)->vertex(2),
-						 pCircle);
-	    if (il == 0)
-	      {
-		Pq_previous_l = m_Pq_element;
-		Pq_element_max_l = m_Pq_element;
-		il++;
-	      }
-	    else if (il == 1)
-	      {
-		Pq_current_l =
-		  m_Pq_element;il++;
-	      }
-	    else
-	      {
-		Pq_next_l = m_Pq_element;
-		if (Pq_element_max_l.fourth.second <= Pq_next_l.fourth.second) 
-		  Pq_element_max_l = Pq_next_l;
-		if ((Pq_current_l.fourth.second>=Pq_previous_l.fourth.second)
-		    &&(Pq_current_l.fourth.second>=Pq_next_l.fourth.second))
-		  {
-		    pq.push(Pq_current_l);
-		  }
-		Pq_previous_l = Pq_current_l;
-		Pq_current_l = Pq_next_l;
-		il++;
-	      }
-	  }
+  Point_2 p0 = pVertex_handle->point();
+  Point_2 c = m_DT.circumcenter(pFace_handle);
+  FT fDist = distance(p0,c);
+  bool b = vector_field_2.is_in_domain(c) && (fDist >= fSepStl_seed);
+  if (b)
+    {
+      Circle pCircle(c,fDist);
+      Pq_element m_Pq_element = Pq_element(
+             pFace_handle->vertex(0),
+             pFace_handle->vertex(1),
+             pFace_handle->vertex(2),
+             pCircle);
+      if (ir == 0)
+        {
+    Pq_previous_r = m_Pq_element;
+    Pq_element_max_r = m_Pq_element;
+    ir++;
+        }
+      else if (ir == 1)
+        {
+    Pq_current_r =
+      m_Pq_element;ir++;
+        }
+      else
+        {
+    Pq_next_r = m_Pq_element;
+    if (Pq_element_max_r.fourth.second <= Pq_next_r.fourth.second)
+      Pq_element_max_r = Pq_next_r;
+    if ((Pq_current_r.fourth.second>=Pq_previous_r.fourth.second)
+        &&(Pq_current_r.fourth.second>=Pq_next_r.fourth.second))
+      {
+        pq.push(Pq_current_r);
+      }
+    Pq_previous_r = Pq_current_r;
+    Pq_current_r = Pq_next_r;
+    ir++;
+        }
+    }
+  p0 = pFace_handle->neighbor(iIndex)->vertex(0)->point();
+  c = m_DT.circumcenter(pFace_handle->neighbor(iIndex));
+  fDist = distance(p0,c);
+  b = vector_field_2.is_in_domain(c) && (fDist >= fSepStl_seed);
+  if (b)
+    {
+      Circle pCircle(c,fDist);
+      Pq_element m_Pq_element = Pq_element(
+             pFace_handle->neighbor(iIndex)->vertex(0),
+             pFace_handle->neighbor(iIndex)->vertex(1),
+             pFace_handle->neighbor(iIndex)->vertex(2),
+             pCircle);
+      if (il == 0)
+        {
+    Pq_previous_l = m_Pq_element;
+    Pq_element_max_l = m_Pq_element;
+    il++;
+        }
+      else if (il == 1)
+        {
+    Pq_current_l =
+      m_Pq_element;il++;
+        }
+      else
+        {
+    Pq_next_l = m_Pq_element;
+    if (Pq_element_max_l.fourth.second <= Pq_next_l.fourth.second) 
+      Pq_element_max_l = Pq_next_l;
+    if ((Pq_current_l.fourth.second>=Pq_previous_l.fourth.second)
+        &&(Pq_current_l.fourth.second>=Pq_next_l.fourth.second))
+      {
+        pq.push(Pq_current_l);
+      }
+    Pq_previous_l = Pq_current_l;
+    Pq_current_l = Pq_next_l;
+    il++;
+        }
+    }
       }
     if ((ir+il == (int) size_-2)&&(size_>2))
       {
-	pq.push(Pq_element_max_l);
-	pq.push(Pq_element_max_r);
+  pq.push(Pq_element_max_l);
+  pq.push(Pq_element_max_r);
       }
   }
   else{
@@ -739,22 +739,22 @@ pq_elements(const Vector_field_2 & vector_field_2, Vertex_container_2 stl_vertic
     Face_circulator pEnd = pFace_handle;
     CGAL_For_all(pFace_handle,pEnd)
       {
-	Point_2 p0 = pFace_handle->vertex(0)->point();
-	Point_2 c = m_DT.circumcenter(pFace_handle);
-	bool b = vector_field_2.is_in_domain(c);
-	if (b){
-	  FT fDist = distance(p0,c);
-	  if (fDist >= fSepStl_seed)
-	    {
-	      Circle pCircle(c,fDist);
-	      Pq_element m_Pq_element = Pq_element(
-						   pFace_handle->vertex(0),
-						   pFace_handle->vertex(1),
-						   pFace_handle->vertex(2),
-						   pCircle);
-	      pq.push(m_Pq_element);
-	    }
-	}
+  Point_2 p0 = pFace_handle->vertex(0)->point();
+  Point_2 c = m_DT.circumcenter(pFace_handle);
+  bool b = vector_field_2.is_in_domain(c);
+  if (b){
+    FT fDist = distance(p0,c);
+    if (fDist >= fSepStl_seed)
+      {
+        Circle pCircle(c,fDist);
+        Pq_element m_Pq_element = Pq_element(
+               pFace_handle->vertex(0),
+               pFace_handle->vertex(1),
+               pFace_handle->vertex(2),
+               pCircle);
+        pq.push(m_Pq_element);
+      }
+  }
       }
   }
 }
@@ -764,13 +764,14 @@ template <class VectorField_2, class Integrator_2>
 inline 
 bool
 Stream_lines_2<VectorField_2, Integrator_2>::get_next_seed_point(FT &
-								 distance, Point_2 & seed_point)
+                 distance, Point_2 & seed_point_)
 {
   Vertex_handle v0, v1, v2; 
   Face_handle fr;
   bool b0,b;
   Pq_element m_Pq_element;
   do{
+    assert(!pq.empty());
     m_Pq_element = pq.top();
     v0 = m_Pq_element.first;
     v1 = m_Pq_element.second;
@@ -779,13 +780,13 @@ Stream_lines_2<VectorField_2, Integrator_2>::get_next_seed_point(FT &
     pq.pop();
     b0 = m_DT.is_face(v0,v1,v2,fr);
     if (b0){
-      seed_point = m_Pq_element.fourth.first;}
+      seed_point_ = m_Pq_element.fourth.first;}
     b = (!pq.empty());
   }while ((b)&&(!b0));
   Biggest_circle = m_Pq_element;
   return b;
 }
-	
+  
 template <class VectorField_2, class Integrator_2>
 typename Stream_lines_2<VectorField_2, Integrator_2>::Stream_line_iterator_2
 Stream_lines_2<VectorField_2, Integrator_2>::begin()
@@ -806,11 +807,11 @@ void Stream_lines_2<VectorField_2, Integrator_2>::make_iterator()
 {
   iterator_container.clear();
   for(typename Stream_line_container_2::iterator
-	begin=stl_container.begin();
+  begin=stl_container.begin();
       begin!=stl_container.end();begin++)
     {
       std::pair<Point_iterator_2, Point_iterator_2>
-	iterator_pair((*begin).begin(), (*begin).end());
+  iterator_pair((*begin).begin(), (*begin).end());
       iterator_container.push_front(iterator_pair);
     }
   begin_iterator = iterator_container.begin();
@@ -823,35 +824,34 @@ template <class VectorField_2, class Integrator_2>
 void
 Stream_lines_2<VectorField_2, Integrator_2>::print_stream_lines_eps(std::ofstream & fw)
 {
-  typename Stream_line_container_2::iterator begin_iterator;
+  typename Stream_line_container_2::iterator it;
   Stream_line_container_2 stl_container_temp = stl_container;
-  Point_container_2 stl;
   fw << "%!PS-Adobe-2.0 EPSF-2.0\n";
   fw << "%%BoundingBox: 0 0" << " " << max_x - min_x << " " << max_y - min_y << "\n";
   fw << "gsave\n";
   fw << "/L {moveto lineto stroke} bind def\n";
   fw << 0.5 << " setlinewidth\n";
   fw << 0.0 << " " << 0.0 << " " << 0.0 << " setrgbcolor\n";
-	
-  for(begin_iterator=stl_container_temp.begin();begin_iterator!=stl_container_temp.end();begin_iterator++)
+  
+  for(it=stl_container_temp.begin(); it!=stl_container_temp.end(); ++it)
     {
-      typename Point_container_2::iterator begin_point_iterator = (*begin_iterator).begin();
-      typename Point_container_2::iterator end_point_iterator = (*begin_iterator).end();
-		
+      typename Point_container_2::iterator begin_point_iterator = (*it).begin();
+      typename Point_container_2::iterator end_point_iterator = (*it).end();
+    
       FT i_prec = (*begin_point_iterator).x() - min_x;
       FT j_prec = (*begin_point_iterator).y() - min_y;
       begin_point_iterator++;
 
       FT i , j;
       for(;begin_point_iterator!=end_point_iterator;begin_point_iterator++)
-	{
-	  Point_2 p = *begin_point_iterator;
-	  i = p.x() - min_x;
-	  j = p.y() - min_y;
-	  fw << i_prec << " " << j_prec << " " << i << " " << j << " L\n";
-	  i_prec = i;
-	  j_prec = j;
-	}
+  {
+    Point_2 p = *begin_point_iterator;
+    i = p.x() - min_x;
+    j = p.y() - min_y;
+    fw << i_prec << " " << j_prec << " " << i << " " << j << " L\n";
+    i_prec = i;
+    j_prec = j;
+  }
     }
   fw << "grestore\n";
   fw << "showpage\n";
@@ -863,22 +863,21 @@ template <class VectorField_2, class Integrator_2>
 void 
 Stream_lines_2<VectorField_2, Integrator_2>::print_stream_lines(std::ofstream & fw)
 {
-  typename Stream_line_container_2::iterator begin_iterator;
+  typename Stream_line_container_2::iterator it;
   Stream_line_container_2 stl_container_temp = stl_container;
-  Point_container_2 stl;
   fw << max_x - min_x << " " << max_y - min_y << "\n";
   fw << stl_container.size() << "\n";
-  for(begin_iterator=stl_container_temp.begin();begin_iterator!=stl_container_temp.end();begin_iterator++)
+  for(it=stl_container_temp.begin(); it!=stl_container_temp.end(); ++it)
     {
-      fw << (*begin_iterator).size() << "\n";
-      typename Point_container_2::iterator begin_point_iterator = (*begin_iterator).begin();
-      typename Point_container_2::iterator end_point_iterator = (*begin_iterator).end();
+      fw << (*it).size() << "\n";
+      typename Point_container_2::iterator begin_point_iterator = (*it).begin();
+      typename Point_container_2::iterator end_point_iterator = (*it).end();
       FT i , j;
       for(;begin_point_iterator!=end_point_iterator;begin_point_iterator++){
-	Point_2 p = *begin_point_iterator;
-	i = p.x() - min_x;
-	j = p.y() - min_y;
-	fw << i << " " << j << "\n";
+  Point_2 p = *begin_point_iterator;
+  i = p.x() - min_x;
+  j = p.y() - min_y;
+  fw << i << " " << j << "\n";
       }
     }
   fw.close();
@@ -902,7 +901,7 @@ Stream_lines_2<VectorField_2, Integrator_2>::get_pq()
       bool b0 = m_DT.is_face(v0,v1,v2,fr);
       Point_2 sdPoint = m_Pq_element.fourth.first;
       if (b0)
-	_list.push_front(sdPoint);
+  _list.push_front(sdPoint);
     }
   return _list;
 }
