@@ -1,4 +1,4 @@
-// Copyright (c) 2006  INRIA Sophia-Antipolis (France).
+// Copyright (c) 2006-2007  INRIA Sophia-Antipolis (France).
 // All rights reserved.
 //
 // This file is part of CGAL (www.cgal.org); you may redistribute it under
@@ -11,16 +11,15 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $Source: 
-// $Revision: 30642 $ $Date: 2006-04-18 14:42:52 +0200 (Tue, 18 Apr 2006) $
-// $Name:  $
+// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.3-branch/Surface_mesher/include/CGAL/Surface_mesh_traits_generator_3.h $
+// $Id: Surface_mesh_traits_generator_3.h 37876 2007-04-03 13:59:17Z lrineau $
 //
 // Author(s)     : Laurent Rineau
 
 #ifndef CGAL_SURFACE_MESH_TRAITS_GENERATOR_3_H
 #define CGAL_SURFACE_MESH_TRAITS_GENERATOR_3_H
 
-#include <CGAL/Surface_mesher/Oracles/Sphere_oracle_3.h>
+#include <CGAL/Surface_mesher/Sphere_oracle_3.h>
 
 namespace CGAL {
 
