@@ -11,8 +11,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.2-branch/Arrangement_2/include/CGAL/Sweep_line_2.h $
-// $Id: Sweep_line_2.h 29531 2006-03-15 19:51:29Z efif $
+// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.3-branch/Arrangement_2/include/CGAL/Sweep_line_2.h $
+// $Id: Sweep_line_2.h 35514 2006-12-11 15:34:13Z wein $
 // 
 //
 // Author(s)     : Baruch Zukerman <baruchzu@post.tau.ac.il>
@@ -83,14 +83,16 @@ class Sweep_line_2 : public Basic_sweep_line_2<Traits_,
 public:
 
   typedef Traits_                                 Traits;
-  typedef typename Traits::Point_2                Point_2;
-  typedef typename Traits::X_monotone_curve_2     X_monotone_curve_2;
-
   typedef Basic_sweep_line_2<Traits,
                              SweepVisitor,
                              CurveWrap,
                              SweepEvent,
                              Allocator>           Base;
+  typedef typename Base::Traits_adaptor           Traits_adaptor;
+  typedef typename Traits_adaptor::Point_2                Point_2;
+  typedef typename Traits_adaptor::X_monotone_curve_2     X_monotone_curve_2;
+
+ 
 
 
   typedef SweepEvent                              Event;
@@ -185,8 +187,6 @@ public:
   { 
     CGAL_PRINT("Handling left curve" << std::endl;);
 
-   
-
     this->m_is_event_on_above = false;
 
     if(! this->m_currentEvent->has_left_curves())
@@ -200,13 +200,9 @@ public:
       //           \
       */
       CGAL_PRINT(" - handling special case " << std::endl;);
-             
-       const std::pair<StatusLineIter, bool>& pair_res = 
-         this->m_statusLine.find_lower (this->m_currentEvent->get_point(), 
-                                        this->m_statusLineCurveLess);
-       this->m_status_line_insert_hint = pair_res.first;
-       this->m_is_event_on_above = pair_res.second;
+      this->_handle_event_without_left_curves();
 
+     
       StatusLineIter temp = this->m_status_line_insert_hint;
 
       if(this->m_is_event_on_above)
@@ -232,7 +228,7 @@ public:
         this->m_currentEvent->set_weak_intersection();
         this->m_visitor->update_event(this->m_currentEvent, sc);
         this->m_currentEvent->add_curve_to_left(sc);
- 
+  
         bool       is_overlap = _add_curve_to_right(this->m_currentEvent, sc);
 
         this->m_traits->split_2_object() (last_curve,
@@ -248,6 +244,7 @@ public:
           this->m_statusLine.erase (temp);
           return;
         }
+        
       }
       else // no left curves for sure
       {
@@ -317,8 +314,9 @@ public:
   /*! Handle the subcurve to the left of the current event point. */
   void _handle_right_curves()
   {
-    CGAL_PRINT("Handling right curves (" ;)
-    CGAL_PRINT(this->m_currentEvent->get_point() << ")\n";)
+    CGAL_PRINT("Handling right curves (" ;);
+    CGAL_SL_DEBUG(PrintEvent(m_currentEvent););
+    CGAL_PRINT(")\n";);
     
     if(! this->m_currentEvent->has_right_curves())
       return;
@@ -364,11 +362,10 @@ public:
       CGAL_SL_DEBUG(PrintStatusLine(););
     
       //BZBZ
-      if(reinterpret_cast<Event*>((*currentOne)->get_left_event()) ==
-          this->m_currentEvent ||
-          reinterpret_cast<Event*>((*prevOne)->get_left_event()) ==
-          this->m_currentEvent ) 
-          _intersect(*prevOne, *currentOne);
+      // if the two curves were neighbours before, we dont need to intersect them again
+      if(!this->m_currentEvent->are_left_neighbours(static_cast<Subcurve*>(*currentOne),
+                                                    static_cast<Subcurve*>(*prevOne)))
+        _intersect(*prevOne, *currentOne);
       prevOne = currentOne;
       ++currentOne;
     }        
@@ -440,7 +437,7 @@ public:
   void _handle_overlap(Event* event, Subcurve* curve, EventCurveIter iter, bool overlap_exist);
   
   /*! Compute intersections between the two given curves. */ 
-  void _intersect(Subcurve *c1, Subcurve *c2, bool after_remove = false);
+  void _intersect(Subcurve *c1, Subcurve *c2);
 
   /*! Remove a curve from the status line. */
   void _remove_curve_from_status_line (Subcurve *leftCurve,
@@ -469,15 +466,6 @@ protected:
   /*! Auxiliary varibales (for splitting curves). */
   X_monotone_curve_2  sub_cv1;
   X_monotone_curve_2  sub_cv2;
-
- 
-
-  template<class SweepCurve>
-  Point_2 get_left_end(SweepCurve* sc) const
-  {
-    return (this->m_traits->construct_min_vertex_2_object()
-            (sc->get_last_curve()));
-  }
 };
 
 
@@ -530,8 +518,7 @@ _remove_curve_from_status_line(Subcurve *leftCurve, bool remove_for_good)
     
     // intersect *next with  *prev 
     _intersect(static_cast<Subcurve*>(*prev),
-               static_cast<Subcurve*>(*next),
-               true);
+               static_cast<Subcurve*>(*next));
   }
   this->m_statusLine.erase(sliter);
   CGAL_PRINT("remove_curve_from_status_line Done\n";)
@@ -559,16 +546,19 @@ void Sweep_line_2<Traits_,
                   CurveWrap,
                   SweepEvent,
                   Allocator>::
- _intersect(Subcurve *c1, Subcurve *c2, bool after_remove)
+ _intersect(Subcurve *c1, Subcurve *c2)
 {
-  CGAL_PRINT("Looking for intersection between:\n\t";)
-  CGAL_SL_DEBUG(c1->Print();)
-  CGAL_PRINT("\t";)
-  CGAL_SL_DEBUG(c2->Print();)
-  CGAL_PRINT("\n";)
+  CGAL_PRINT("Looking for intersection between:\n\t";);
+  CGAL_SL_DEBUG(c1->Print(););
+  CGAL_PRINT("\t";);
+  CGAL_SL_DEBUG(c2->Print(););
+  CGAL_PRINT("\n";);
 
   CGAL_assertion(c1 != c2);
-  
+ 
+  if(this->m_currentEvent->is_plus_boundary_in_x())
+    return;
+
   // look up for (c1,c2) in the table and insert if doesnt exist
   CurvesPair curves_pair(c1,c2);
   if(! (m_curves_pair_set.insert(curves_pair)).second )
@@ -600,7 +590,8 @@ void Sweep_line_2<Traits_,
   if((SweepEvent*)c1->get_left_event() == this->m_currentEvent &&
      (SweepEvent*)c2->get_left_event() == this->m_currentEvent)
   {
-     ++vi;
+    CGAL_PRINT(" [Skipping common left endpoint...]\n";);
+    ++vi;
   }
 
   //BZBZ
@@ -611,26 +602,26 @@ void Sweep_line_2<Traits_,
   if (reinterpret_cast<SweepEvent*>(c1->get_right_event()) ==
       reinterpret_cast<SweepEvent*>(c2->get_right_event()))
   {
+    CGAL_PRINT(" [Skipping common right endpoint...]\n";);
     --vi_end; 
   }  
 
   const std::pair<Point_2,unsigned int>  *xp_point;
-
-  if (after_remove)
+  if(vi != vi_end)
   {
-    for( ; vi != vi_end ; ++vi)
+    xp_point = object_cast<std::pair<Point_2,unsigned int> > (&(*vi));
+    if(xp_point != NULL && this->m_currentEvent->is_finite())
     {
-      xp_point = object_cast<std::pair<Point_2,unsigned int> > (&(*vi));
-      CGAL_assertion (xp_point != NULL);
-
       if (this->m_traits->compare_xy_2_object()
           (this->m_currentEvent->get_point(),
-           xp_point->first) ==  SMALLER)
+           xp_point->first) !=  SMALLER)
       {
-        break;
+        CGAL_PRINT(" [Skipping an event smaller that the status line...]\n";);
+        ++vi;
       }
     }
   }
+   
 
   for( ; vi != vi_end ; ++vi)
   {
@@ -847,39 +838,52 @@ _handle_overlap(Event* event,
       overlap_cv = object_cast<X_monotone_curve_2> (obj_vec.front());
     }
 
-    // Get the left end of overlap_cv
-    Point_2 begin_overlap =
-      this->m_traits->construct_min_vertex_2_object()(overlap_cv);
-    // Get the right end of overlap_cv
-    Point_2 end_overlap =
-      this->m_traits->construct_max_vertex_2_object()(overlap_cv);
+   
+    // Get the right end of overlap_cv (if it is bounded from the right).
+    Event* right_end;
 
-    const std::pair<Event*, bool>& pair_res =
-      push_event(end_overlap, Base_event::OVERLAP);
-    //find the event assiciated with end_overlap point (right end point)
-    //EventQueueIter q_iter = this->m_queue->find( end_overlap );
-
-    //TODO: in polylines, the event point of the right-end of the overlap
-    // is not have to be exist yet,
-    //CGAL_assertion(q_iter != this->m_queue->end());
-
-    //Event* right_end = (*q_iter).second;
-
-    Event* right_end = pair_res.first;
-
-    if (this->m_traits->compare_xy_2_object() (event->get_point(), 
-                                               begin_overlap) != EQUAL)
+    if(this->m_traits->boundary_in_x_2_object()(overlap_cv,
+                                                MAX_END) != NO_BOUNDARY ||
+       this->m_traits->boundary_in_y_2_object()(overlap_cv,
+                                                MAX_END) != NO_BOUNDARY)
     {
-      this->m_traits->split_2_object() (overlap_cv,
-                                        event->get_point(), 
-                                        sub_cv1, sub_cv2);
-      overlap_cv = sub_cv2;
+      CGAL_assertion((*iter)->get_right_event() == curve->get_right_event());
+       right_end = (Event*)curve->get_right_event();
     }
+    else
+    {
+      Point_2 end_overlap =
+        this->m_traits->construct_max_vertex_2_object()(overlap_cv);
+
+      const std::pair<Event*, bool>& pair_res =
+        push_event(end_overlap, Base_event::OVERLAP);
+      right_end = pair_res.first;
+    }
+
+     // Get the left end of overlap_cv (if it is bounded from the left).
+     if(this->m_traits->boundary_in_x_2_object()(overlap_cv,
+                                                 MIN_END) == NO_BOUNDARY &&
+        this->m_traits->boundary_in_y_2_object()(overlap_cv,
+                                                 MIN_END) == NO_BOUNDARY)
+     {
+       Point_2 begin_overlap =
+         this->m_traits->construct_min_vertex_2_object()(overlap_cv);
+       if (this->m_traits->compare_xy_2_object() (event->get_point(), 
+                                                  begin_overlap) != EQUAL)
+       {
+         this->m_traits->split_2_object() (overlap_cv,
+                                           event->get_point(), 
+                                           sub_cv1, sub_cv2);
+         overlap_cv = sub_cv2;
+       }
+     }
 
     // Alocate a new Subcure for the overlap
     Subcurve *overlap_sc = this->m_subCurveAlloc.allocate(1);
     this->m_subCurveAlloc.construct(overlap_sc, this->m_masterSubcurve);
-    overlap_sc->init(overlap_cv, event, right_end );
+    overlap_sc->init(overlap_cv);
+    overlap_sc->set_left_event(event);
+    overlap_sc->set_right_event(right_end);
     m_overlap_subCurves.push_back(overlap_sc);
 
     CGAL_PRINT(curve<<" + " <<*iter<<" => " <<overlap_sc<<"\n");
@@ -899,10 +903,14 @@ _handle_overlap(Event* event,
 
     //BZBZ 07.09.05
     if((Event*)curve->get_right_event() != right_end)
+    {
       _add_curve_to_right(right_end, curve);
+    }
 
     if((Event*)(*iter)->get_right_event() != right_end)
+    {
       _add_curve_to_right(right_end, (*iter));
+    }
 
     // Replace current sub-curve (*iter) with the new sub-curve
     (*iter) = overlap_sc;
