@@ -15,8 +15,8 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.2-branch/Circulator/include/CGAL/circulator.h $
-// $Id: circulator.h 28567 2006-02-16 14:30:13Z lsaboret $
+// $URL: svn+ssh://scm.gforge.inria.fr/svn/cgal/branches/CGAL-3.3-branch/Circulator/include/CGAL/circulator.h $
+// $Id: circulator.h 37166 2007-03-16 17:36:19Z afabri $
 // 
 //
 // Author(s)     : Lutz Kettner  <kettner@inf.ethz.ch>
@@ -1012,27 +1012,30 @@ private:
     I m_begin;
     I m_end;
     I current;
+    bool empty;
 
     // The following static iterator is needed so that we have a value
     // that can be uniquely compared (the default constructed one can be
     // different each time).
-    static I null_iterator;
+  //static I null_iterator;
 
 public:
 // CREATION
 
-    Circulator_from_iterator() : m_begin( null_iterator),
-                                 m_end(   null_iterator),
-                                 current( null_iterator) {}
+    Circulator_from_iterator() : m_begin(),
+                                 m_end(),
+                                 current(),
+				 empty( true)
+  {}
 
     Circulator_from_iterator( const I& bgn, const I& end)
-        : m_begin(bgn), m_end(end), current(bgn) {}
+        : m_begin(bgn), m_end(end), current(bgn), empty(bgn==end) {}
 
     Circulator_from_iterator( const I& bgn, const I& end, const I& cur)
-        : m_begin(bgn), m_end(end), current(cur) {}
+        : m_begin(bgn), m_end(end), current(cur), empty(bgn==end) {}
 
     Circulator_from_iterator( const Self& c, const I& cur)
-        : m_begin( c.m_begin), m_end( c.m_end), current(cur) {}
+        : m_begin( c.m_begin), m_end( c.m_end), current(cur), empty(c.empty) {}
 
 
     template <class II, class A1, class A2, class A3>
@@ -1041,18 +1044,17 @@ public:
     Circulator_from_iterator(
         const Circulator_from_iterator<II,A1,A2,A3>& ii)
     : m_begin( ii.begin()), m_end( ii.end()),
-        current(ii.current_iterator()) {}
+        current(ii.current_iterator()), empty(ii.begin()==ii.end()) {}
 
 //
 // OPERATIONS
 
     bool operator==( CGAL_NULL_TYPE p) const {
         CGAL_assertion( p == CGAL_CIRC_NULL);
-        CGAL_assertion((m_end == m_begin) || (current != m_end));
-        return m_end == m_begin;
+        return empty;
     }
     bool operator!=( CGAL_NULL_TYPE p) const { return !(*this == p); }
-    bool operator==( const Self& c) const { return current == c.current;}
+    bool operator==( const Self& c) const { return  current == c.current;}
     bool operator!=( const Self& c) const { return !(*this == c); }
     reference  operator*() const {
         CGAL_assertion( current != m_end);
@@ -1122,8 +1124,8 @@ public:
     Self      min_circulator()   const { return Self( m_begin, m_end); }
 };
 
-template < class I, class  T, class Size, class Dist>
-I Circulator_from_iterator< I, T, Size, Dist>::null_iterator = I();
+//template < class I, class  T, class Size, class Dist>
+//I Circulator_from_iterator< I, T, Size, Dist>::null_iterator = I();
 
 template < class D, class I, class  T, class Size, class Dist> inline
 Circulator_from_iterator< I, T, Size, Dist>
